/**
 * 
 */
package org.springblade.mcp.controller;

import java.time.Duration;
import java.util.UUID;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.constant.TokenConstant;
import org.springblade.core.log.annotation.ApiLog;
import org.springblade.core.redis.cache.BladeRedis;
import org.springblade.core.secure.BladeUser;
import org.springblade.core.secure.properties.JwtProperties;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.secure.utils.JwtUtil;
import org.springblade.core.secure.utils.WebUtil;
import org.springblade.core.tool.exception.ServiceException;
import org.springblade.core.tool.support.Kv;
import org.springblade.core.tool.utils.Func;
import org.springblade.mcp.entity.McpUser;
import org.springblade.mcp.service.McpUserService;
import org.springblade.mcp.utils.DigestUtil;
import org.springblade.mcp.utils.TokenUtil;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.wf.captcha.SpecCaptcha;

import cn.hutool.core.util.StrUtil;
import io.jsonwebtoken.Claims;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.AllArgsConstructor;

/**
 * @author wookvn
 * 
 */
@RestController
@AllArgsConstructor
@RequestMapping("/auth")
@Api(value = "用户登录授权认证", tags = "登录授权接口")
public class AuthController extends BladeController {
	
	private final BladeRedis bladeRedis;
	private final JwtProperties jwtProperties;
	private final McpUserService mcpUserService;
	
	private final static String CAPTCHA_HEADER_KEY = "Captcha-Key";
	private final static String CAPTCHA_HEADER_CODE = "Captcha-Code";
	private final static String CAPTCHA_KEY = "mcp:auth::platomix:captcha:";
	
	/**
	 * 前端需要传递几个参数【该模式与BI登录一致】：
	 * 
	 * 1、header
	 *   1.1. Authorization[必须] 
	 *     值="Basic base64Encode(clientId:clientSecret)"；
	 *     该值意义：根据clientId查询表blade_client获取token过期时间，否则默认使用saber:saber_secret
	 *   1.2. Captcha-Code  当grant_type=captcha时必选项
	 *   1.3. Captcha-Key   当grant_type=captcha时必选项
	 *   1.4. 其他非登录相关接口，鉴权需要传递token，headerKey=@Value("${blade.token.headerName:Blade-Auth}")
	 * 2、params
	 *   2.1. grant_type[必须, captcha/refresh_token] 
	 *   2.2. username  当grant_type=captcha时必选项
	 *   2.3. password  当grant_type=captcha时必选项
	 *   
	 * @param username
	 * @param password
	 * @return
	 */
	@ApiLog("登录用户验证")
	@PostMapping("/token")
	@ApiOperation(value = "获取认证令牌", notes = "账号:account,密码:password")
	public Kv token(HttpServletRequest request,
					@ApiParam(value = "账号", required = true) @RequestParam(required = false) String username,
					@ApiParam(value = "密码", required = true) @RequestParam(required = false) String password) {
		Kv authInfo = Kv.create();
		
		String grantType = request.getParameter("grant_type");
		String refreshToken = request.getParameter("refresh_token");
		McpUser mcpUser = null;
		//验证码登录
		if(StrUtil.isBlank(grantType) || "captcha".equalsIgnoreCase(grantType)) {
			String key = request.getHeader(CAPTCHA_HEADER_KEY);
			String code = request.getHeader(CAPTCHA_HEADER_CODE);
			// 获取验证码
			String redisCode = bladeRedis.get(CAPTCHA_KEY + key);
			// 判断验证码
			if (code == null || !StrUtil.equalsIgnoreCase(redisCode, code)) {
				throw new ServiceException("验证码不正确");
			}
			// 验证通过删除key
			bladeRedis.del(CAPTCHA_KEY + key);
			/**
			 * 查询数据库获取登录用户信息
			 * 安全性考虑：
			 * 1.要求前端对密码做MD5加密后传输，后端登录校验前先使用 {@link org.springblade.mcp.utils.DigestUtil#hex(String)}，再进行比对;
			 * 2.数据库存储的密码规则： {@link org.springblade.mcp.utils.DigestUtil#encrypt(String)}。
			 *  对密码明文，先md5加密后，再sha1加密 【 sha1Hex(md5Hex("密码明文")) 】。
			 */
			mcpUser = mcpUserService.login(username, DigestUtil.hex(password));
			if (mcpUser == null) {
				return authInfo.set("error_code", HttpServletResponse.SC_BAD_REQUEST).set("error_description", "用户名或密码不正确");
			}
			return TokenUtil.createAuthInfo(mcpUser);
		}else if("refresh_token".equalsIgnoreCase(grantType) && StrUtil.isNotBlank(refreshToken)) { //刷新token
			Claims claims = AuthUtil.parseJWT(refreshToken);
			if (claims != null) {
				String tokenType = Func.toStr(claims.get(TokenConstant.TOKEN_TYPE));
				if (tokenType.equals(TokenConstant.REFRESH_TOKEN)) {
					// 获取用户信息
					int userId = Func.toInt(claims.get(TokenConstant.USER_ID));
					mcpUser = mcpUserService.getById(userId);
				}
			}
			if (mcpUser == null) {
				return authInfo.set("error_code", HttpServletResponse.SC_BAD_REQUEST).set("error_description", "refreshToken不正确");
			}
			return TokenUtil.createAuthInfo(mcpUser);
		}else {
			return authInfo.set("error_code", HttpServletResponse.SC_BAD_REQUEST).set("error_description", "无效的请求");
		}
	}
	
	/**
	 * 
	 * @return
	 */
	@GetMapping("/logout")
	@ApiOperation(value = "退出登录")
	public Kv logout() {
		BladeUser user = AuthUtil.getUser();
		if (user != null && jwtProperties.getState()) {
			String token = JwtUtil.getToken(WebUtil.getRequest().getHeader(TokenConstant.HEADER));
			JwtUtil.removeAccessToken(user.getTenantId(), String.valueOf(user.getUserId()), token);
		}
		return Kv.create().set("success", "true").set("msg", "success");
	}
	
	/**
	 * 
	 * @return
	 */
	@GetMapping("/captcha")
	@ApiOperation(value = "获取验证码")
	public Kv captcha() {
		SpecCaptcha specCaptcha = new SpecCaptcha(130, 48, 5);
		String verCode = specCaptcha.text().toLowerCase();
		String key = UUID.randomUUID().toString();
		// 存入redis并设置过期时间为30分钟
		bladeRedis.setEx(CAPTCHA_KEY + key, verCode, Duration.ofMinutes(30));
		// 将key和base64返回给前端
		return Kv.create().set("key", key).set("image", specCaptcha.toBase64());
	}
}
